@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,7 +15,7 @@
 
 import os
 import random
-from gettext import gettext as _
+from swift import gettext_ as _
 from logging import DEBUG
 from math import sqrt
 from time import time, ctime
@@ -24,11 +24,12 @@ from eventlet import GreenPool, sleep, Timeout
 
 import swift.common.db
 from swift.account.server import DATADIR
-from swift.common.db import AccountBroker
+from swift.account.backend import AccountBroker
 from swift.common.direct_client import ClientException, \
     direct_delete_container, direct_delete_object, direct_get_container
 from swift.common.ring import Ring
-from swift.common.utils import get_logger, whataremyips, config_true_value
+from swift.common.utils import get_logger, whataremyips, ismount, \
+    config_true_value
 from swift.common.daemon import Daemon
 
 
@@ -120,7 +121,7 @@ class AccountReaper(Daemon):
         begin = time()
         try:
             for device in os.listdir(self.devices):
-                if self.mount_check and not os.path.ismount(
+                if self.mount_check and not ismount(
                         os.path.join(self.devices, device)):
                     self.logger.increment('errors')
                     self.logger.debug(
@@ -206,7 +207,7 @@ class AccountReaper(Daemon):
 
         .. seealso::
 
-            :class:`swift.common.db.AccountBroker` for the broker class.
+            :class:`swift.account.backend.AccountBroker` for the broker class.
 
         .. seealso::
 
@@ -267,7 +268,7 @@ class AccountReaper(Daemon):
                 self.stats_objects_possibly_remaining
         if self.stats_return_codes:
             log += _(', return codes: ')
-            for code in sorted(self.stats_return_codes.keys()):
+            for code in sorted(self.stats_return_codes):
                 log += '%s %sxxs, ' % (self.stats_return_codes[code], code)
             log = log[:-2]
         log += _(', elapsed: %.02fs') % (time() - begin)
@@ -331,7 +332,7 @@ class AccountReaper(Daemon):
                 self.stats_return_codes[2] = \
                     self.stats_return_codes.get(2, 0) + 1
                 self.logger.increment('return_codes.2')
-            except ClientException, err:
+            except ClientException as err:
                 if self.logger.getEffectiveLevel() <= DEBUG:
                     self.logger.exception(
                         _('Exception with %(ip)s:%(port)s/%(device)s'), node)
@@ -374,7 +375,7 @@ class AccountReaper(Daemon):
                 self.stats_return_codes[2] = \
                     self.stats_return_codes.get(2, 0) + 1
                 self.logger.increment('return_codes.2')
-            except ClientException, err:
+            except ClientException as err:
                 if self.logger.getEffectiveLevel() <= DEBUG:
                     self.logger.exception(
                         _('Exception with %(ip)s:%(port)s/%(device)s'), node)
@@ -433,7 +434,7 @@ class AccountReaper(Daemon):
                 self.stats_return_codes[2] = \
                     self.stats_return_codes.get(2, 0) + 1
                 self.logger.increment('return_codes.2')
-            except ClientException, err:
+            except ClientException as err:
                 if self.logger.getEffectiveLevel() <= DEBUG:
                     self.logger.exception(
                         _('Exception with %(ip)s:%(port)s/%(device)s'), node)
