@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -30,11 +30,15 @@ class SwiftException(Exception):
     pass
 
 
-class AuditException(SwiftException):
+class DiskFileError(SwiftException):
     pass
 
 
-class DiskFileError(SwiftException):
+class DiskFileNotOpen(DiskFileError):
+    pass
+
+
+class DiskFileQuarantined(DiskFileError):
     pass
 
 
@@ -46,6 +50,17 @@ class DiskFileNotExist(DiskFileError):
     pass
 
 
+class DiskFileDeleted(DiskFileNotExist):
+
+    def __init__(self, metadata=None):
+        self.metadata = metadata or {}
+        self.timestamp = self.metadata.get('X-Timestamp', 0)
+
+
+class DiskFileExpired(DiskFileDeleted):
+    pass
+
+
 class DiskFileNoSpace(DiskFileError):
     pass
 
@@ -110,3 +125,11 @@ class ListingIterNotAuthorized(ListingIterError):
 
 class SegmentError(SwiftException):
     pass
+
+
+class ReplicationException(Exception):
+    pass
+
+
+class ReplicationLockTimeout(LockTimeout):
+    pass
