@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,7 +13,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import with_statement
 import functools
 import errno
 import os
@@ -22,7 +21,7 @@ import signal
 import time
 import subprocess
 import re
-from gettext import gettext as _
+from swift import gettext_ as _
 
 from swift.common.utils import search_tree, remove_file, write_file
 
@@ -63,7 +62,8 @@ def setup_env():
         print _("WARNING: Unable to increase file descriptor limit.  "
                 "Running as non-root?")
 
-    os.environ['PYTHON_EGG_CACHE'] = '/tmp'
+    # Set PYTHON_EGG_CACHE if it isn't already set
+    os.environ.setdefault('PYTHON_EGG_CACHE', '/tmp')
 
 
 def command(func):
@@ -99,7 +99,7 @@ def watch_server_pids(server_pids, interval=1, **kwargs):
                 try:
                     # let pid stop if it wants to
                     os.waitpid(pid, os.WNOHANG)
-                except OSError, e:
+                except OSError as e:
                     if e.errno not in (errno.ECHILD, errno.ESRCH):
                         raise  # else no such child/process
             # check running pids for server
@@ -442,7 +442,7 @@ class Server():
                     print _('Signal %s  pid: %s  signal: %s') % (self.server,
                                                                  pid, sig)
                 os.kill(pid, sig)
-            except OSError, e:
+            except OSError as e:
                 if e.errno == errno.ESRCH:
                     # pid does not exist
                     if kwargs.get('verbose'):
@@ -610,11 +610,13 @@ class Server():
             print '%s...(%s)' % (msg, conf_file)
             try:
                 pid = self.spawn(conf_file, **kwargs)
-            except OSError, e:
+            except OSError as e:
                 if e.errno == errno.ENOENT:
                     #TODO(clayg): should I check if self.cmd exists earlier?
                     print _("%s does not exist") % self.cmd
                     break
+                else:
+                    raise
             pids[pid] = conf_file
 
         return pids
