@@ -48,8 +48,8 @@ post -m quota-bytes:
 
 from swift.common.swob import HTTPForbidden, HTTPRequestEntityTooLarge, \
     HTTPBadRequest, wsgify
-
-from swift.proxy.controllers.base import get_account_info
+from swift.common.utils import register_swift_info
+from swift.proxy.controllers.base import get_account_info, get_object_info
 
 
 class AccountQuotaMiddleware(object):
@@ -64,16 +64,26 @@ class AccountQuotaMiddleware(object):
     @wsgify
     def __call__(self, request):
 
-        if request.method not in ("POST", "PUT"):
+        if request.method not in ("POST", "PUT", "COPY"):
             return self.app
 
         try:
-            request.split_path(2, 4, rest_with_last=True)
+            ver, account, container, obj = request.split_path(
+                2, 4, rest_with_last=True)
         except ValueError:
             return self.app
 
-        new_quota = request.headers.get('X-Account-Meta-Quota-Bytes')
-        remove_quota = request.headers.get('X-Remove-Account-Meta-Quota-Bytes')
+        if not container:
+            # account request, so we pay attention to the quotas
+            new_quota = request.headers.get(
+                'X-Account-Meta-Quota-Bytes')
+            remove_quota = request.headers.get(
+                'X-Remove-Account-Meta-Quota-Bytes')
+        else:
+            # container or object request; even if the quota headers are set
+            # in the request, they're meaningless
+            new_quota = remove_quota = None
+
         if remove_quota:
             new_quota = 0    # X-Remove dominates if both are present
 
@@ -86,13 +96,36 @@ class AccountQuotaMiddleware(object):
         if new_quota is not None:
             return HTTPForbidden()
 
+        if obj and request.method == "POST" or not obj:
+            return self.app
+
+        if request.method == 'COPY':
+            copy_from = container + '/' + obj
+        else:
+            copy_from = request.headers.get('X-Copy-From')
+
+        content_length = (request.content_length or 0)
+
         account_info = get_account_info(request.environ, self.app)
         if not account_info or not account_info['bytes']:
             return self.app
-        new_size = int(account_info['bytes']) + (request.content_length or 0)
-        quota = int(account_info['meta'].get('quota-bytes', -1))
+        try:
+            quota = int(account_info['meta'].get('quota-bytes', -1))
+        except ValueError:
+            return self.app
+        if quota < 0:
+            return self.app
 
-        if 0 <= quota < new_size:
+        if obj and copy_from:
+            path = '/' + ver + '/' + account + '/' + copy_from.lstrip('/')
+            object_info = get_object_info(request.environ, self.app, path)
+            if not object_info or not object_info['length']:
+                content_length = 0
+            else:
+                content_length = int(object_info['length'])
+
+        new_size = int(account_info['bytes']) + content_length
+        if quota < new_size:
             return HTTPRequestEntityTooLarge()
 
         return self.app
@@ -100,6 +133,8 @@ class AccountQuotaMiddleware(object):
 
 def filter_factory(global_conf, **local_conf):
     """Returns a WSGI filter app for use with paste.deploy."""
+    register_swift_info('account_quotas')
+
     def account_quota_filter(app):
         return AccountQuotaMiddleware(app)
     return account_quota_filter
