@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,7 +15,7 @@
 
 import errno
 import os
-from gettext import gettext as _
+from swift import gettext_ as _
 
 from swift import __version__ as swiftver
 from swift.common.swob import Request, Response
@@ -201,7 +201,14 @@ class ReconMiddleware(object):
         """get disk utilization statistics"""
         devices = []
         for entry in os.listdir(self.devices):
-            if check_mount(self.devices, entry):
+            try:
+                mounted = check_mount(self.devices, entry)
+            except OSError as err:
+                devices.append({'device': entry, 'mounted': str(err),
+                                'size': '', 'used': '', 'avail': ''})
+                continue
+
+            if mounted:
                 path = os.path.join(self.devices, entry)
                 disk = os.statvfs(path)
                 capacity = disk.f_bsize * disk.f_blocks
@@ -228,7 +235,7 @@ class ReconMiddleware(object):
                             md5sum.update(block)
                             block = f.read(4096)
                     sums[ringfile] = md5sum.hexdigest()
-                except IOError, err:
+                except IOError as err:
                     sums[ringfile] = None
                     if err.errno != errno.ENOENT:
                         self.logger.exception(_('Error reading ringfile'))
