@@ -13,8 +13,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import with_statement
-
 import os
 import time
 import traceback
@@ -54,8 +52,8 @@ class ContainerController(object):
     save_headers = ['x-container-read', 'x-container-write',
                     'x-container-sync-key', 'x-container-sync-to']
 
-    def __init__(self, conf):
-        self.logger = get_logger(conf, log_route='container-server')
+    def __init__(self, conf, logger=None):
+        self.logger = logger or get_logger(conf, log_route='container-server')
         self.root = conf.get('devices', '/srv/node/')
         self.mount_check = config_true_value(conf.get('mount_check', 'true'))
         self.node_timeout = int(conf.get('node_timeout', 3))
@@ -257,7 +255,7 @@ class ContainerController(object):
                     broker.initialize(timestamp)
                     created = True
                 except DatabaseAlreadyExists:
-                    pass
+                    created = False
             else:
                 created = broker.is_deleted()
                 broker.update_put_timestamp(timestamp)
