@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,7 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from gettext import gettext as _
+from swift import gettext_ as _
 from time import ctime, time
 from random import random, shuffle
 from struct import unpack_from
@@ -24,9 +24,9 @@ import swift.common.db
 from swift.container import server as container_server
 from swiftclient import ClientException, delete_object, put_object, \
     quote
+from swift.container.backend import ContainerBroker
 from swift.common.direct_client import direct_get_object
 from swift.common.ring import Ring
-from swift.common.db import ContainerBroker
 from swift.common.utils import audit_location_generator, get_logger, \
     hash_path, config_true_value, validate_sync_to, whataremyips, FileLikeIter
 from swift.common.daemon import Daemon
@@ -147,7 +147,7 @@ class ContainerSync(Daemon):
         swift.common.db.DB_PREALLOCATION = \
             config_true_value(conf.get('db_preallocation', 'f'))
 
-    def run_forever(self):
+    def run_forever(self, *args, **kwargs):
         """
         Runs container sync scans until stopped.
         """
@@ -167,7 +167,7 @@ class ContainerSync(Daemon):
             if elapsed < self.interval:
                 sleep(self.interval - elapsed)
 
-    def run_once(self):
+    def run_once(self, *args, **kwargs):
         """
         Runs a single container sync scan.
         """
@@ -245,7 +245,7 @@ class ContainerSync(Daemon):
                 if err:
                     self.logger.info(
                         _('ERROR %(db_file)s: %(validate_sync_to_err)s'),
-                        {'db_file': broker.db_file,
+                        {'db_file': str(broker),
                          'validate_sync_to_err': err})
                     self.container_failures += 1
                     self.logger.increment('failures')
@@ -299,7 +299,7 @@ class ContainerSync(Daemon):
             self.container_failures += 1
             self.logger.increment('failures')
             self.logger.exception(_('ERROR Syncing %s'),
-                                  broker.db_file if broker else path)
+                                  broker if broker else path)
 
     def container_sync_row(self, row, sync_to, sync_key, broker, info):
         """
@@ -323,7 +323,7 @@ class ContainerSync(Daemon):
                                   headers={'x-timestamp': row['created_at'],
                                            'x-container-sync-key': sync_key},
                                   proxy=self.proxy)
-                except ClientException, err:
+                except ClientException as err:
                     if err.http_status != HTTP_NOT_FOUND:
                         raise
                 self.container_deletes += 1
@@ -348,14 +348,14 @@ class ContainerSync(Daemon):
                             timestamp = this_timestamp
                             headers = these_headers
                             body = this_body
-                    except ClientException, err:
+                    except ClientException as err:
                         # If any errors are not 404, make sure we report the
                         # non-404 one. We don't want to mistakenly assume the
                         # object no longer exists just because one says so and
                         # the others errored for some other reason.
                         if not exc or exc.http_status == HTTP_NOT_FOUND:
                             exc = err
-                    except (Exception, Timeout), err:
+                    except (Exception, Timeout) as err:
                         exc = err
                 if timestamp < looking_for_timestamp:
                     if exc:
@@ -380,7 +380,7 @@ class ContainerSync(Daemon):
                 self.container_puts += 1
                 self.logger.increment('puts')
                 self.logger.timing_since('puts.timing', start_time)
-        except ClientException, err:
+        except ClientException as err:
             if err.http_status == HTTP_UNAUTHORIZED:
                 self.logger.info(
                     _('Unauth %(sync_from)r => %(sync_to)r'),
@@ -397,14 +397,14 @@ class ContainerSync(Daemon):
             else:
                 self.logger.exception(
                     _('ERROR Syncing %(db_file)s %(row)s'),
-                    {'db_file': broker.db_file, 'row': row})
+                    {'db_file': str(broker), 'row': row})
             self.container_failures += 1
             self.logger.increment('failures')
             return False
-        except (Exception, Timeout), err:
+        except (Exception, Timeout) as err:
             self.logger.exception(
                 _('ERROR Syncing %(db_file)s %(row)s'),
-                {'db_file': broker.db_file, 'row': row})
+                {'db_file': str(broker), 'row': row})
             self.container_failures += 1
             self.logger.increment('failures')
             return False
