@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -28,6 +28,8 @@ from nose import SkipTest
 from xml.dom import minidom
 from swiftclient import get_auth
 
+from test import safe_repr
+
 
 class AuthenticationFailed(Exception):
     pass
@@ -38,16 +40,28 @@ class RequestError(Exception):
 
 
 class ResponseError(Exception):
-    def __init__(self, response):
+    def __init__(self, response, method, path):
         self.status = response.status
         self.reason = response.reason
-        Exception.__init__(self)
+        self.method = method
+        self.path = path
+        self.headers = response.getheaders()
+
+        for name, value in self.headers:
+            if name.lower() == 'x-trans-id':
+                self.txid = value
+                break
+        else:
+            self.txid = None
+
+        super(ResponseError, self).__init__()
 
     def __str__(self):
-        return '%d: %s' % (self.status, self.reason)
+        return repr(self)
 
     def __repr__(self):
-        return '%d: %s' % (self.status, self.reason)
+        return '%d: %r (%r %r) txid=%s' % (
+            self.status, self.reason, self.method, self.path, self.txid)
 
 
 def listing_empty(method):
@@ -131,12 +145,10 @@ class Connection(object):
         auth_netloc = "%s:%d" % (self.auth_host, self.auth_port)
         auth_url = auth_scheme + auth_netloc + auth_path
 
-        (storage_url, storage_token) = get_auth(auth_url,
-              auth_user, self.password,
-              snet=False,
-              tenant_name=self.account,
-              auth_version=self.auth_version,
-              os_options={})
+        (storage_url, storage_token) = get_auth(
+            auth_url, auth_user, self.password, snet=False,
+            tenant_name=self.account, auth_version=self.auth_version,
+            os_options={})
 
         if not (storage_url and storage_token):
             raise AuthenticationFailed()
@@ -155,7 +167,11 @@ class Connection(object):
         self.storage_host = x[2].split(':')[0]
         if ':' in x[2]:
             self.storage_port = int(x[2].split(':')[1])
-        self.storage_url = '/%s/%s' % (x[3], x[4])
+        # Make sure storage_url is a string and not unicode, since
+        # keystoneclient (called by swiftclient) returns them in
+        # unicode and this would cause troubles when doing
+        # no_safe_quote query.
+        self.storage_url = str('/%s/%s' % (x[3], x[4]))
 
         self.storage_token = storage_token
 
@@ -214,18 +230,22 @@ class Connection(object):
 
         self.response = None
         try_count = 0
+        fail_messages = []
         while try_count < 5:
             try_count += 1
 
             try:
                 self.response = try_request()
-            except httplib.HTTPException:
+            except httplib.HTTPException as e:
+                fail_messages.append(safe_repr(e))
                 continue
 
             if self.response.status == 401:
+                fail_messages.append("Response 401")
                 self.authenticate()
                 continue
             elif self.response.status == 503:
+                fail_messages.append("Response 503")
                 if try_count != 5:
                     time.sleep(5)
                 continue
@@ -235,7 +255,11 @@ class Connection(object):
         if self.response:
             return self.response.status
 
-        raise RequestError('Unable to complete http request')
+        request = "{method} {path} headers: {headers} data: {data}".format(
+            method=method, path=path, headers=headers, data=data)
+        raise RequestError('Unable to complete http request: %s. '
+                           'Attempts: %s, Failures: %s' %
+                           (request, len(fail_messages), fail_messages))
 
     def put_start(self, path, hdrs={}, parms={}, cfg={}, chunked=False):
         self.http_connect()
@@ -255,10 +279,6 @@ class Connection(object):
                           for (x, y) in parms.items()]
             path = '%s?%s' % (path, '&'.join(query_args))
 
-            query_args = ['%s=%s' % (urllib.quote(x),
-                          urllib.quote(str(y))) for (x, y) in parms.items()]
-            path = '%s?%s' % (path, '&'.join(query_args))
-
         self.connection = self.conn_class(self.storage_host,
                                           port=self.storage_port)
         #self.connection.set_debuglevel(3)
@@ -310,21 +330,21 @@ class Account(Base):
         return Container(self.conn, self.name, container_name)
 
     def containers(self, hdrs={}, parms={}, cfg={}):
-        format = parms.get('format', None)
-        if format not in [None, 'json', 'xml']:
-            raise RequestError('Invalid format: %s' % format)
-        if format is None and 'format' in parms:
+        format_type = parms.get('format', None)
+        if format_type not in [None, 'json', 'xml']:
+            raise RequestError('Invalid format: %s' % format_type)
+        if format_type is None and 'format' in parms:
             del parms['format']
 
         status = self.conn.make_request('GET', self.path, hdrs=hdrs,
                                         parms=parms, cfg=cfg)
         if status == 200:
-            if format == 'json':
+            if format_type == 'json':
                 conts = json.loads(self.conn.response.read())
                 for cont in conts:
                     cont['name'] = cont['name'].encode('utf-8')
                 return conts
-            elif format == 'xml':
+            elif format_type == 'xml':
                 conts = []
                 tree = minidom.parseString(self.conn.response.read())
                 for x in tree.getElementsByTagName('container'):
@@ -344,7 +364,8 @@ class Account(Base):
         elif status == 204:
             return []
 
-        raise ResponseError(self.conn.response)
+        raise ResponseError(self.conn.response, 'GET',
+                            self.conn.make_path(self.path))
 
     def delete_containers(self):
         for c in listing_items(self.containers):
@@ -358,7 +379,8 @@ class Account(Base):
         if self.conn.make_request('HEAD', self.path, hdrs=hdrs,
                                   parms=parms, cfg=cfg) != 204:
 
-            raise ResponseError(self.conn.response)
+            raise ResponseError(self.conn.response, 'HEAD',
+                                self.conn.make_path(self.path))
 
         fields = [['object_count', 'x-account-object-count'],
                   ['container_count', 'x-account-container-count'],
@@ -387,8 +409,8 @@ class Container(Base):
 
     def delete_files(self):
         for f in listing_items(self.files):
-            file = self.file(f)
-            if not file.delete():
+            file_item = self.file(f)
+            if not file_item.delete():
                 return False
 
         return listing_empty(self.files)
@@ -400,37 +422,39 @@ class Container(Base):
         return File(self.conn, self.account, self.name, file_name)
 
     def files(self, hdrs={}, parms={}, cfg={}):
-        format = parms.get('format', None)
-        if format not in [None, 'json', 'xml']:
-            raise RequestError('Invalid format: %s' % format)
-        if format is None and 'format' in parms:
+        format_type = parms.get('format', None)
+        if format_type not in [None, 'json', 'xml']:
+            raise RequestError('Invalid format: %s' % format_type)
+        if format_type is None and 'format' in parms:
             del parms['format']
 
         status = self.conn.make_request('GET', self.path, hdrs=hdrs,
                                         parms=parms, cfg=cfg)
         if status == 200:
-            if format == 'json':
+            if format_type == 'json':
                 files = json.loads(self.conn.response.read())
 
-                for file in files:
-                    file['name'] = file['name'].encode('utf-8')
-                    file['content_type'] = file['content_type'].encode('utf-8')
+                for file_item in files:
+                    file_item['name'] = file_item['name'].encode('utf-8')
+                    file_item['content_type'] = file_item['content_type'].\
+                        encode('utf-8')
                 return files
-            elif format == 'xml':
+            elif format_type == 'xml':
                 files = []
                 tree = minidom.parseString(self.conn.response.read())
                 for x in tree.getElementsByTagName('object'):
-                    file = {}
+                    file_item = {}
                     for key in ['name', 'hash', 'bytes', 'content_type',
                                 'last_modified']:
 
-                        file[key] = x.getElementsByTagName(key)[0].\
+                        file_item[key] = x.getElementsByTagName(key)[0].\
                             childNodes[0].nodeValue
-                    files.append(file)
+                    files.append(file_item)
 
-                for file in files:
-                    file['name'] = file['name'].encode('utf-8')
-                    file['content_type'] = file['content_type'].encode('utf-8')
+                for file_item in files:
+                    file_item['name'] = file_item['name'].encode('utf-8')
+                    file_item['content_type'] = file_item['content_type'].\
+                        encode('utf-8')
                 return files
             else:
                 content = self.conn.response.read()
@@ -444,11 +468,12 @@ class Container(Base):
         elif status == 204:
             return []
 
-        raise ResponseError(self.conn.response)
+        raise ResponseError(self.conn.response, 'GET',
+                            self.conn.make_path(self.path))
 
     def info(self, hdrs={}, parms={}, cfg={}):
-        status = self.conn.make_request('HEAD', self.path, hdrs=hdrs,
-                                        parms=parms, cfg=cfg)
+        self.conn.make_request('HEAD', self.path, hdrs=hdrs,
+                               parms=parms, cfg=cfg)
 
         if self.conn.response.status == 204:
             fields = [['bytes_used', 'x-container-bytes-used'],
@@ -456,7 +481,8 @@ class Container(Base):
 
             return self.header_fields(fields)
 
-        raise ResponseError(self.conn.response)
+        raise ResponseError(self.conn.response, 'HEAD',
+                            self.conn.make_path(self.path))
 
     @property
     def path(self):
@@ -531,7 +557,8 @@ class File(Base):
         if self.conn.make_request('DELETE', self.path, hdrs=hdrs,
                                   parms=parms) != 204:
 
-            raise ResponseError(self.conn.response)
+            raise ResponseError(self.conn.response, 'DELETE',
+                                self.conn.make_path(self.path))
 
         return True
 
@@ -539,7 +566,8 @@ class File(Base):
         if self.conn.make_request('HEAD', self.path, hdrs=hdrs,
                                   parms=parms, cfg=cfg) != 200:
 
-            raise ResponseError(self.conn.response)
+            raise ResponseError(self.conn.response, 'HEAD',
+                                self.conn.make_path(self.path))
 
         fields = [['content_length', 'content-length'],
                   ['content_type', 'content-type'],
@@ -559,7 +587,8 @@ class File(Base):
         if status == 404:
             return False
         elif (status < 200) or (status > 299):
-            raise ResponseError(self.conn.response)
+            raise ResponseError(self.conn.response, 'HEAD',
+                                self.conn.make_path(self.path))
 
         for hdr in self.conn.response.getheaders():
             if hdr[0].lower() == 'content-type':
@@ -594,20 +623,21 @@ class File(Base):
         return data
 
     def read(self, size=-1, offset=0, hdrs=None, buffer=None,
-             callback=None, cfg={}):
+             callback=None, cfg={}, parms={}):
 
         if size > 0:
-            range = 'bytes=%d-%d' % (offset, (offset + size) - 1)
+            range_string = 'bytes=%d-%d' % (offset, (offset + size) - 1)
             if hdrs:
-                hdrs['Range'] = range
+                hdrs['Range'] = range_string
             else:
-                hdrs = {'Range': range}
+                hdrs = {'Range': range_string}
 
         status = self.conn.make_request('GET', self.path, hdrs=hdrs,
-                                        cfg=cfg)
+                                        cfg=cfg, parms=parms)
 
-        if(status < 200) or (status > 299):
-            raise ResponseError(self.conn.response)
+        if (status < 200) or (status > 299):
+            raise ResponseError(self.conn.response, 'GET',
+                                self.conn.make_path(self.path))
 
         for hdr in self.conn.response.getheaders():
             if hdr[0].lower() == 'content-type':
@@ -630,8 +660,9 @@ class File(Base):
     def read_md5(self):
         status = self.conn.make_request('GET', self.path)
 
-        if(status < 200) or (status > 299):
-            raise ResponseError(self.conn.response)
+        if (status < 200) or (status > 299):
+            raise ResponseError(self.conn.response, 'GET',
+                                self.conn.make_path(self.path))
 
         checksum = hashlib.md5()
 
@@ -664,7 +695,8 @@ class File(Base):
             self.conn.make_request('POST', self.path, hdrs=headers, cfg=cfg)
 
             if self.conn.response.status not in (201, 202):
-                raise ResponseError(self.conn.response)
+                raise ResponseError(self.conn.response, 'POST',
+                                    self.conn.make_path(self.path))
 
         return True
 
@@ -717,13 +749,18 @@ class File(Base):
                     callback(transferred, self.size)
 
             self.conn.put_end()
-        except socket.timeout, err:
+        except socket.timeout as err:
             raise err
 
         if (self.conn.response.status < 200) or \
            (self.conn.response.status > 299):
-            raise ResponseError(self.conn.response)
+            raise ResponseError(self.conn.response, 'PUT',
+                                self.conn.make_path(self.path))
 
+        try:
+            data.seek(0)
+        except IOError:
+            pass
         self.md5 = self.compute_md5sum(data)
 
         return True
@@ -731,6 +768,7 @@ class File(Base):
     def write_random(self, size=None, hdrs={}, parms={}, cfg={}):
         data = self.random_data(size)
         if not self.write(data, hdrs=hdrs, parms=parms, cfg=cfg):
-            raise ResponseError(self.conn.response)
+            raise ResponseError(self.conn.response, 'PUT',
+                                self.conn.make_path(self.path))
         self.md5 = self.compute_md5sum(StringIO.StringIO(data))
         return data
