@@ -1,5 +1,5 @@
 #!/usr/bin/python -u
-# Copyright (c) 2010-2012 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,8 +16,9 @@
 
 import os
 import shutil
+import time
 
-from subprocess import call, Popen
+from subprocess import call
 from unittest import main, TestCase
 from uuid import uuid4
 
@@ -34,7 +35,7 @@ class TestEmptyDevice(TestCase):
     def setUp(self):
         (self.pids, self.port2server, self.account_ring, self.container_ring,
          self.object_ring, self.url, self.token,
-         self.account) = reset_environment()
+         self.account, self.configs) = reset_environment()
 
     def tearDown(self):
         kill_servers(self.port2server, self.pids)
@@ -42,8 +43,7 @@ class TestEmptyDevice(TestCase):
     def _get_objects_dir(self, onode):
         device = onode['device']
         node_id = (onode['port'] - 6000) / 10
-        obj_server_conf = readconf('/etc/swift/object-server/%s.conf' %
-                                   node_id)
+        obj_server_conf = readconf(self.configs['object'] % node_id)
         devices = obj_server_conf['app:object-server']['devices']
         obj_dir = '%s/%s' % (devices, device)
         return obj_dir
@@ -105,31 +105,49 @@ class TestEmptyDevice(TestCase):
                 client.get_container(self.url, self.token, container)[1]]
         if obj not in objs:
             raise Exception('Container listing did not know about object')
-        for cnode in cnodes:
-            objs = [o['name'] for o in
-                    direct_client.direct_get_container(
-                        cnode, cpart, self.account, container)[1]]
-            if obj not in objs:
-                raise Exception(
-                    'Container server %s:%s did not know about object' %
-                    (cnode['ip'], cnode['port']))
+        timeout = time.time() + 5
+        found_objs_on_cnode = []
+        while time.time() < timeout:
+            for cnode in [c for c in cnodes if cnodes not in
+                          found_objs_on_cnode]:
+                objs = [o['name'] for o in
+                        direct_client.direct_get_container(
+                            cnode, cpart, self.account, container)[1]]
+                if obj in objs:
+                    found_objs_on_cnode.append(cnode)
+            if len(found_objs_on_cnode) >= len(cnodes):
+                break
+            time.sleep(0.3)
+        if len(found_objs_on_cnode) < len(cnodes):
+            missing = ['%s:%s' % (cnode['ip'], cnode['port']) for cnode in
+                       cnodes if cnode not in found_objs_on_cnode]
+            raise Exception('Container servers %r did not know about object' %
+                            missing)
         start_server(onode['port'], self.port2server, self.pids)
         self.assertFalse(os.path.exists(obj_dir))
         exc = None
         try:
             direct_client.direct_get_object(onode, opart, self.account,
                                             container, obj)
-        except direct_client.ClientException, err:
+        except direct_client.ClientException as err:
             exc = err
         self.assertEquals(exc.http_status, 404)
         self.assertFalse(os.path.exists(obj_dir))
 
+        try:
+            port_num = onode['replication_port']
+        except KeyError:
+            port_num = onode['port']
+        try:
+            another_port_num = another_onode['replication_port']
+        except KeyError:
+            another_port_num = another_onode['port']
         call(['swift-object-replicator',
-              '/etc/swift/object-server/%d.conf' %
-              ((onode['port'] - 6000) / 10), 'once'])
+              self.configs['object-replicator'] %
+              ((port_num - 6000) / 10), 'once'])
         call(['swift-object-replicator',
-              '/etc/swift/object-server/%d.conf' %
-              ((another_onode['port'] - 6000) / 10), 'once'])
+              self.configs['object-replicator'] %
+              ((another_port_num - 6000) / 10), 'once'])
 
         odata = direct_client.direct_get_object(onode, opart, self.account,
                                                 container, obj)[-1]
@@ -140,7 +158,7 @@ class TestEmptyDevice(TestCase):
         try:
             direct_client.direct_get_object(another_onode, opart, self.account,
                                             container, obj)
-        except direct_client.ClientException, err:
+        except direct_client.ClientException as err:
             exc = err
         self.assertEquals(exc.http_status, 404)
 
