@@ -146,6 +146,55 @@ class TestTempURL(unittest.TestCase):
         self.assertEquals(req.environ['swift.authorize_override'], True)
         self.assertEquals(req.environ['REMOTE_USER'], '.wsgi.tempurl')
 
+    def test_head_valid(self):
+        method = 'HEAD'
+        expires = int(time() + 86400)
+        path = '/v1/a/c/o'
+        key = 'abc'
+        hmac_body = '%s\n%s\n%s' % (method, expires, path)
+        sig = hmac.new(key, hmac_body, sha1).hexdigest()
+        req = self._make_request(path, keys=[key], environ={
+            'REQUEST_METHOD': 'HEAD',
+            'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s'
+            % (sig, expires)})
+        self.tempurl.app = FakeApp(iter([('200 Ok', (), '123')]))
+        resp = req.get_response(self.tempurl)
+        self.assertEquals(resp.status_int, 200)
+
+    def test_get_valid_with_filename_and_inline(self):
+        method = 'GET'
+        expires = int(time() + 86400)
+        path = '/v1/a/c/o'
+        key = 'abc'
+        hmac_body = '%s\n%s\n%s' % (method, expires, path)
+        sig = hmac.new(key, hmac_body, sha1).hexdigest()
+        req = self._make_request(path, keys=[key], environ={
+            'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s&'
+            'filename=bob%%20%%22killer%%22.txt&inline=' % (sig, expires)})
+        self.tempurl.app = FakeApp(iter([('200 Ok', (), '123')]))
+        resp = req.get_response(self.tempurl)
+        self.assertEquals(resp.status_int, 200)
+        self.assertEquals(resp.headers['content-disposition'], 'inline')
+        self.assertEquals(req.environ['swift.authorize_override'], True)
+        self.assertEquals(req.environ['REMOTE_USER'], '.wsgi.tempurl')
+
+    def test_get_valid_with_inline(self):
+        method = 'GET'
+        expires = int(time() + 86400)
+        path = '/v1/a/c/o'
+        key = 'abc'
+        hmac_body = '%s\n%s\n%s' % (method, expires, path)
+        sig = hmac.new(key, hmac_body, sha1).hexdigest()
+        req = self._make_request(path, keys=[key], environ={
+            'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s&'
+            'inline=' % (sig, expires)})
+        self.tempurl.app = FakeApp(iter([('200 Ok', (), '123')]))
+        resp = req.get_response(self.tempurl)
+        self.assertEquals(resp.status_int, 200)
+        self.assertEquals(resp.headers['content-disposition'], 'inline')
+        self.assertEquals(req.environ['swift.authorize_override'], True)
+        self.assertEquals(req.environ['REMOTE_USER'], '.wsgi.tempurl')
+
     def test_obj_trailing_slash(self):
         method = 'GET'
         expires = int(time() + 86400)
@@ -214,6 +263,7 @@ class TestTempURL(unittest.TestCase):
         resp = req.get_response(self.tempurl)
         self.assertEquals(resp.status_int, 401)
         self.assertTrue('Temp URL invalid' in resp.body)
+        self.assertTrue('Www-Authenticate' in resp.headers)
 
     def test_put_valid(self):
         method = 'PUT'
@@ -246,6 +296,7 @@ class TestTempURL(unittest.TestCase):
         resp = req.get_response(self.tempurl)
         self.assertEquals(resp.status_int, 401)
         self.assertTrue('Temp URL invalid' in resp.body)
+        self.assertTrue('Www-Authenticate' in resp.headers)
 
     def test_missing_sig(self):
         method = 'GET'
@@ -260,6 +311,7 @@ class TestTempURL(unittest.TestCase):
         resp = req.get_response(self.tempurl)
         self.assertEquals(resp.status_int, 401)
         self.assertTrue('Temp URL invalid' in resp.body)
+        self.assertTrue('Www-Authenticate' in resp.headers)
 
     def test_missing_expires(self):
         method = 'GET'
@@ -274,6 +326,7 @@ class TestTempURL(unittest.TestCase):
         resp = req.get_response(self.tempurl)
         self.assertEquals(resp.status_int, 401)
         self.assertTrue('Temp URL invalid' in resp.body)
+        self.assertTrue('Www-Authenticate' in resp.headers)
 
     def test_bad_path(self):
         method = 'GET'
@@ -289,6 +342,7 @@ class TestTempURL(unittest.TestCase):
         resp = req.get_response(self.tempurl)
         self.assertEquals(resp.status_int, 401)
         self.assertTrue('Temp URL invalid' in resp.body)
+        self.assertTrue('Www-Authenticate' in resp.headers)
 
     def test_no_key(self):
         method = 'GET'
@@ -304,6 +358,7 @@ class TestTempURL(unittest.TestCase):
         resp = req.get_response(self.tempurl)
         self.assertEquals(resp.status_int, 401)
         self.assertTrue('Temp URL invalid' in resp.body)
+        self.assertTrue('Www-Authenticate' in resp.headers)
 
     def test_head_allowed_by_get(self):
         method = 'GET'
@@ -356,6 +411,7 @@ class TestTempURL(unittest.TestCase):
                          sig, expires)})
         resp = req.get_response(self.tempurl)
         self.assertEquals(resp.status_int, 401)
+        self.assertTrue('Www-Authenticate' in resp.headers)
 
     def test_post_not_allowed(self):
         method = 'POST'
@@ -372,6 +428,7 @@ class TestTempURL(unittest.TestCase):
         resp = req.get_response(self.tempurl)
         self.assertEquals(resp.status_int, 401)
         self.assertTrue('Temp URL invalid' in resp.body)
+        self.assertTrue('Www-Authenticate' in resp.headers)
 
     def test_delete_not_allowed(self):
         method = 'DELETE'
@@ -388,6 +445,7 @@ class TestTempURL(unittest.TestCase):
         resp = req.get_response(self.tempurl)
         self.assertEquals(resp.status_int, 401)
         self.assertTrue('Temp URL invalid' in resp.body)
+        self.assertTrue('Www-Authenticate' in resp.headers)
 
     def test_delete_allowed_with_conf(self):
         self.tempurl.methods.append('DELETE')
@@ -420,6 +478,7 @@ class TestTempURL(unittest.TestCase):
         resp = req.get_response(self.tempurl)
         self.assertEquals(resp.status_int, 401)
         self.assertTrue('Temp URL invalid' in resp.body)
+        self.assertTrue('Www-Authenticate' in resp.headers)
 
     def test_changed_path_invalid(self):
         method = 'GET'
@@ -435,6 +494,7 @@ class TestTempURL(unittest.TestCase):
         resp = req.get_response(self.tempurl)
         self.assertEquals(resp.status_int, 401)
         self.assertTrue('Temp URL invalid' in resp.body)
+        self.assertTrue('Www-Authenticate' in resp.headers)
 
     def test_changed_sig_invalid(self):
         method = 'GET'
@@ -454,6 +514,7 @@ class TestTempURL(unittest.TestCase):
         resp = req.get_response(self.tempurl)
         self.assertEquals(resp.status_int, 401)
         self.assertTrue('Temp URL invalid' in resp.body)
+        self.assertTrue('Www-Authenticate' in resp.headers)
 
     def test_changed_expires_invalid(self):
         method = 'GET'
@@ -469,6 +530,7 @@ class TestTempURL(unittest.TestCase):
         resp = req.get_response(self.tempurl)
         self.assertEquals(resp.status_int, 401)
         self.assertTrue('Temp URL invalid' in resp.body)
+        self.assertTrue('Www-Authenticate' in resp.headers)
 
     def test_different_key_invalid(self):
         method = 'GET'
@@ -484,6 +546,7 @@ class TestTempURL(unittest.TestCase):
         resp = req.get_response(self.tempurl)
         self.assertEquals(resp.status_int, 401)
         self.assertTrue('Temp URL invalid' in resp.body)
+        self.assertTrue('Www-Authenticate' in resp.headers)
 
     def test_removed_incoming_header(self):
         self.tempurl = tempurl.filter_factory({
@@ -600,46 +663,56 @@ class TestTempURL(unittest.TestCase):
             self.tempurl._get_temp_url_info(
                 {'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s' % (
                     s, e)}),
-            (s, e, None))
+            (s, e, None, None))
         self.assertEquals(
             self.tempurl._get_temp_url_info(
                 {'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s&'
                  'filename=bobisyouruncle' % (s, e)}),
-            (s, e, 'bobisyouruncle'))
+            (s, e, 'bobisyouruncle', None))
         self.assertEquals(
             self.tempurl._get_temp_url_info({}),
-            (None, None, None))
+            (None, None, None, None))
         self.assertEquals(
             self.tempurl._get_temp_url_info(
                 {'QUERY_STRING': 'temp_url_expires=%s' % e}),
-            (None, e, None))
+            (None, e, None, None))
         self.assertEquals(
             self.tempurl._get_temp_url_info(
                 {'QUERY_STRING': 'temp_url_sig=%s' % s}),
-            (s, None, None))
+            (s, None, None, None))
         self.assertEquals(
             self.tempurl._get_temp_url_info(
                 {'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=bad' % (
                     s)}),
-            (s, 0, None))
+            (s, 0, None, None))
+        self.assertEquals(
+            self.tempurl._get_temp_url_info(
+                {'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s&'
+                 'inline=' % (s, e)}),
+            (s, e, None, True))
+        self.assertEquals(
+            self.tempurl._get_temp_url_info(
+                {'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s&'
+                 'filename=bobisyouruncle&inline=' % (s, e)}),
+            (s, e, 'bobisyouruncle', True))
         e = int(time() - 1)
         self.assertEquals(
             self.tempurl._get_temp_url_info(
                 {'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s' % (
                     s, e)}),
-            (s, 0, None))
+            (s, 0, None, None))
 
-    def test_get_hmac(self):
+    def test_get_hmacs(self):
         self.assertEquals(
-            self.tempurl._get_hmac(
+            self.tempurl._get_hmacs(
                 {'REQUEST_METHOD': 'GET', 'PATH_INFO': '/v1/a/c/o'},
-                1, 'abc'),
-            '026d7f7cc25256450423c7ad03fc9f5ffc1dab6d')
+                1, ['abc']),
+            ['026d7f7cc25256450423c7ad03fc9f5ffc1dab6d'])
         self.assertEquals(
-            self.tempurl._get_hmac(
+            self.tempurl._get_hmacs(
                 {'REQUEST_METHOD': 'HEAD', 'PATH_INFO': '/v1/a/c/o'},
-                1, 'abc', request_method='GET'),
-            '026d7f7cc25256450423c7ad03fc9f5ffc1dab6d')
+                1, ['abc'], request_method='GET'),
+            ['026d7f7cc25256450423c7ad03fc9f5ffc1dab6d'])
 
     def test_invalid(self):
 
@@ -653,6 +726,26 @@ class TestTempURL(unittest.TestCase):
             self.tempurl._invalid({'REQUEST_METHOD': 'HEAD'},
                                   _start_response)))
 
+    def test_auth_scheme_value(self):
+        # Passthrough
+        environ = {}
+        resp = self._make_request('/v1/a/c/o', environ=environ).get_response(
+            self.tempurl)
+        self.assertEquals(resp.status_int, 401)
+        self.assertTrue('Temp URL invalid' not in resp.body)
+        self.assertTrue('Www-Authenticate' in resp.headers)
+        self.assertTrue('swift.auth_scheme' not in environ)
+
+        # Rejected by TempURL
+        req = self._make_request('/v1/a/c/o', keys=['abc'],
+                                 environ={'REQUEST_METHOD': 'PUT',
+                                 'QUERY_STRING':
+                                 'temp_url_sig=dummy&temp_url_expires=1234'})
+        resp = req.get_response(self.tempurl)
+        self.assertEquals(resp.status_int, 401)
+        self.assertTrue('Temp URL invalid' in resp.body)
+        self.assert_('Www-Authenticate' in resp.headers)
+
     def test_clean_incoming_headers(self):
         irh = ''
         iah = ''
@@ -769,6 +862,12 @@ class TestTempURL(unittest.TestCase):
         self.assertTrue('test-header-yes' not in hdrs)
         self.assertTrue('test-header-yes-this' in hdrs)
 
+    def test_unicode_metadata_value(self):
+        meta = {"temp-url-key": "test", "temp-url-key-2": u"test2"}
+        results = tempurl.get_tempurl_keys_from_metadata(meta)
+        for str_value in results:
+            self.assertTrue(isinstance(str_value, str))
+
 
 if __name__ == '__main__':
     unittest.main()
