@@ -1,4 +1,4 @@
-# Copyright (c) 2010 OpenStack, LLC.
+# Copyright (c) 2010 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,9 +13,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-""" Tests for swift.common.wsgi """
+"""Tests for swift.common.wsgi"""
 
-from __future__ import with_statement
 import errno
 import logging
 import mimetools
@@ -25,6 +24,7 @@ import os
 import pickle
 from textwrap import dedent
 from gzip import GzipFile
+from contextlib import nested
 from StringIO import StringIO
 from collections import defaultdict
 from contextlib import closing
@@ -32,7 +32,11 @@ from urllib import quote
 
 from eventlet import listen
 
-import swift
+import mock
+
+import swift.common.middleware.catch_errors
+import swift.proxy.server
+
 from swift.common.swob import Request
 from swift.common import wsgi, utils, ring
 
@@ -42,21 +46,24 @@ from mock import patch
 
 
 def _fake_rings(tmpdir):
-    with closing(GzipFile(os.path.join(tmpdir, 'account.ring.gz'), 'wb')) as f:
+    account_ring_path = os.path.join(tmpdir, 'account.ring.gz')
+    with closing(GzipFile(account_ring_path, 'wb')) as f:
         pickle.dump(ring.RingData([[0, 1, 0, 1], [1, 0, 1, 0]],
                     [{'id': 0, 'zone': 0, 'device': 'sda1', 'ip': '127.0.0.1',
                       'port': 6012},
                      {'id': 1, 'zone': 1, 'device': 'sdb1', 'ip': '127.0.0.1',
                       'port': 6022}], 30),
                     f)
-    with closing(GzipFile(os.path.join(tmpdir, 'container.ring.gz'), 'wb')) as f:
+    container_ring_path = os.path.join(tmpdir, 'container.ring.gz')
+    with closing(GzipFile(container_ring_path, 'wb')) as f:
         pickle.dump(ring.RingData([[0, 1, 0, 1], [1, 0, 1, 0]],
                     [{'id': 0, 'zone': 0, 'device': 'sda1', 'ip': '127.0.0.1',
                       'port': 6011},
                      {'id': 1, 'zone': 1, 'device': 'sdb1', 'ip': '127.0.0.1',
                       'port': 6021}], 30),
                     f)
-    with closing(GzipFile(os.path.join(tmpdir, 'object.ring.gz'), 'wb')) as f:
+    object_ring_path = os.path.join(tmpdir, 'object.ring.gz')
+    with closing(GzipFile(object_ring_path, 'wb')) as f:
         pickle.dump(ring.RingData([[0, 1, 0, 1], [1, 0, 1, 0]],
                     [{'id': 0, 'zone': 0, 'device': 'sda1', 'ip': '127.0.0.1',
                       'port': 6010},
@@ -66,7 +73,7 @@ def _fake_rings(tmpdir):
 
 
 class TestWSGI(unittest.TestCase):
-    """ Tests for swift.common.wsgi """
+    """Tests for swift.common.wsgi"""
 
     def setUp(self):
         utils.HASH_PATH_PREFIX = 'startcap'
@@ -239,11 +246,13 @@ class TestWSGI(unittest.TestCase):
                     socket.SO_REUSEADDR: 1,
                     socket.SO_KEEPALIVE: 1,
                 },
+                socket.IPPROTO_TCP: {
+                    socket.TCP_NODELAY: 1,
+                }
             }
             if hasattr(socket, 'TCP_KEEPIDLE'):
-                expected_socket_opts[socket.IPPROTO_TCP] = {
-                    socket.TCP_KEEPIDLE: 600,
-                }
+                expected_socket_opts[socket.IPPROTO_TCP][
+                    socket.TCP_KEEPIDLE] = 600
             self.assertEquals(sock.opts, expected_socket_opts)
             # test ssl
             sock = wsgi.get_socket(ssl_conf)
@@ -495,6 +504,40 @@ class TestWSGI(unittest.TestCase):
         self.assertEquals(r.body, 'the body')
         self.assertEquals(r.environ['swift.source'], 'UT')
 
+    def test_run_server_global_conf_callback(self):
+        calls = defaultdict(lambda: 0)
+
+        def _initrp(conf_file, app_section, *args, **kwargs):
+            return (
+                {'__file__': 'test', 'workers': 0},
+                'logger',
+                'log_name')
+
+        def _global_conf_callback(preloaded_app_conf, global_conf):
+            calls['_global_conf_callback'] += 1
+            self.assertEqual(
+                preloaded_app_conf, {'__file__': 'test', 'workers': 0})
+            self.assertEqual(global_conf, {'log_name': 'log_name'})
+            global_conf['test1'] = 'one'
+
+        def _loadapp(uri, name=None, **kwargs):
+            calls['_loadapp'] += 1
+            self.assertTrue('global_conf' in kwargs)
+            self.assertEqual(kwargs['global_conf'],
+                             {'log_name': 'log_name', 'test1': 'one'})
+
+        with nested(
+                mock.patch.object(wsgi, '_initrp', _initrp),
+                mock.patch.object(wsgi, 'get_socket'),
+                mock.patch.object(wsgi, 'drop_privileges'),
+                mock.patch.object(wsgi, 'loadapp', _loadapp),
+                mock.patch.object(wsgi, 'capture_stdio'),
+                mock.patch.object(wsgi, 'run_server')):
+            wsgi.run_wsgi('conf_file', 'app_section',
+                          global_conf_callback=_global_conf_callback)
+        self.assertEqual(calls['_global_conf_callback'], 1)
+        self.assertEqual(calls['_loadapp'], 1)
+
     def test_pre_auth_req_with_empty_env_no_path(self):
         r = wsgi.make_pre_authed_request(
             {}, 'GET')
@@ -535,6 +578,7 @@ class TestWSGI(unittest.TestCase):
         self.assertEquals(r.environ['SCRIPT_NAME'], '')
         self.assertEquals(r.environ['PATH_INFO'], '/override')
 
+
 class TestWSGIContext(unittest.TestCase):
 
     def test_app_call(self):
@@ -554,6 +598,27 @@ class TestWSGIContext(unittest.TestCase):
         self.assertEquals(wc._response_status, '404 Not Found')
         self.assertEquals(''.join(it), 'Ok\n')
 
+    def test_app_iter_is_closable(self):
+
+        def app(env, start_response):
+            start_response('200 OK', [('Content-Length', '25')])
+            yield 'aaaaa'
+            yield 'bbbbb'
+            yield 'ccccc'
+            yield 'ddddd'
+            yield 'eeeee'
+
+        wc = wsgi.WSGIContext(app)
+        r = Request.blank('/')
+        iterable = wc._app_call(r.environ)
+        self.assertEquals(wc._response_status, '200 OK')
+
+        iterator = iter(iterable)
+        self.assertEqual('aaaaa', iterator.next())
+        self.assertEqual('bbbbb', iterator.next())
+        iterable.close()
+        self.assertRaises(StopIteration, iterator.next)
+
 
 if __name__ == '__main__':
     unittest.main()
