@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -33,13 +33,35 @@ class TestContainerController(unittest.TestCase):
         controller = proxy_server.ContainerController(self.app, 'a', 'c')
         with mock.patch('swift.proxy.controllers.base.http_connect',
                         fake_http_connect(200, 200, body='')):
-            req = Request.blank('/a/c', {'PATH_INFO': '/a/c'})
+            req = Request.blank('/v1/a/c', {'PATH_INFO': '/v1/a/c'})
             resp = controller.HEAD(req)
         self.assertEqual(2, resp.status_int // 100)
         self.assertTrue("swift.container/a/c" in resp.environ)
         self.assertEqual(headers_to_container_info(resp.headers),
                          resp.environ['swift.container/a/c'])
 
+    def test_swift_owner(self):
+        owner_headers = {
+            'x-container-read': 'value', 'x-container-write': 'value',
+            'x-container-sync-key': 'value', 'x-container-sync-to': 'value'}
+        controller = proxy_server.ContainerController(self.app, 'a', 'c')
+
+        req = Request.blank('/v1/a/c')
+        with mock.patch('swift.proxy.controllers.base.http_connect',
+                        fake_http_connect(200, 200, headers=owner_headers)):
+            resp = controller.HEAD(req)
+        self.assertEquals(2, resp.status_int // 100)
+        for key in owner_headers:
+            self.assertTrue(key not in resp.headers)
+
+        req = Request.blank('/v1/a/c', environ={'swift_owner': True})
+        with mock.patch('swift.proxy.controllers.base.http_connect',
+                        fake_http_connect(200, 200, headers=owner_headers)):
+            resp = controller.HEAD(req)
+        self.assertEquals(2, resp.status_int // 100)
+        for key in owner_headers:
+            self.assertTrue(key in resp.headers)
+
 
 if __name__ == '__main__':
     unittest.main()
